
import validators
from django.conf import settings

def is_site_url_provided() -> str:
    '''
        Checks if SITE_URL is provided in settings.py
    '''
    if hasattr(settings, 'SITE_URL'):
        if validators.url(settings.SITE_URL):
            if validators.length(settings.SITE_URL, min_val=1, max_val=255):
                return None
            else:
                return "SITE_URL is Token cant be empty or longer than 255 characters"
        else:
            return "SITE_URL is not a valid url"
    else: 
        return "SITE_URL is not provided in settings.py"


def is_static_url_provided() -> str:
    '''
        Checks if STATIC_URL is provided in settings.py
    '''
    if hasattr(settings, 'STATIC_URL'):
        if str(settings.STATIC_URL).startswith('/') and str(settings.STATIC_URL).endswith('/'):
            return None
        else:
            return "STATIC_URL is not a valid url"
    else: 
        return "STATIC_URL is not provided in settings.py"

def is_media_url_provided() -> str:
    '''
        Checks if MEDIA_URL is provided in settings.py
    '''
    if hasattr(settings, 'MEDIA_URL'):
        if str(settings.MEDIA_URL).startswith('/') and str(settings.MEDIA_URL).endswith('/'):
            return None
        else:
            return "MEDIA_URL is not a valid url"
    else: 
        return "MEDIA_URL is not provided in settings.py"

def is_pikutis_api_key_provided() -> str:
    '''
        Checks if PIKUTIS_API_KEY is provided in settings.py
    '''
    if hasattr(settings, 'PIKUTIS_API_KEY'):
        if validators.length(settings.PIKUTIS_API_KEY, min_val=1, max_val=255):
            return None
        else:
            return "PIKUTIS_API_KEY is Token cant be empty or longer than 255 characters"
    else: 
        return "PIKUTIS_API_KEY is not provided in settings.py"
    
def are_settings_valid() -> list:
    '''
        Validates settings.py
    '''
    errors = []
    errors.append(is_site_url_provided())
    errors.append(is_static_url_provided())
    errors.append(is_media_url_provided())
    errors.append(is_pikutis_api_key_provided())
    return list(filter(None, errors))


import json
import logging
from django.http import HttpResponse
from .controllers.api import PdfApiParams, call_pdf_api, get_pdf_file
from .controllers.validators import are_settings_valid
from .controllers.html_transformation import fix_static_urls, fix_media_urls
from typing import Optional
logger = logging.getLogger(__name__)

def get_document_pdf_response(file_string:str, file_name:str = "document", api_params:Optional[PdfApiParams]=None) -> tuple[HttpResponse, bytes]:
    '''
        Returns django http response with pdf file
        and file bytes. File will be named as filename or if not provided: "document.pdf"
    '''
    try:
        file = None
        response = None
        errors:list = []
        errors = are_settings_valid()
        if errors:
            return HttpResponse(json.dumps(errors), status=500), None
        
        file_string = fix_static_urls(file_string)
        file_string = fix_media_urls(file_string)
        
        pdf_response:dict = call_pdf_api(file_string, api_params)
        
        if pdf_response["error"]:
            return HttpResponse(pdf_response["error"], status=500), None

        file = get_pdf_file(pdf_response)
        response = HttpResponse(file, content_type='application/pdf')
        response['Content-Disposition'] = f'inline;filename={file_name}.pdf'
        return (response, file)
    
    except Exception as e:
        return HttpResponse(str(e), status=500), None


import base64
from dataclasses import dataclass
import json
from typing import Optional
import requests
from django.conf import settings

FORMAT_CHOICES = {
    "A4": "A4",
    "A5": "A5",
    "A6": "A6",
    "Letter": "Letter",
}

@dataclass
class PdfApiParams:
    '''
        Class for api params
    '''
    display_header_footer:bool = False
    format:str = FORMAT_CHOICES["A4"]
    landscape:bool = False
    print_background:bool = False
    prefer_css_page_size:bool = True
    scale:float = 1.0
    margin_top:float = 0
    margin_bottom:float = 0
    margin_left:float = 0
    margin_right:float = 0
    auth_token_name:Optional[str] = None
    auth_token_value:Optional[str] = None
    wait_timeout:int = 10

    def to_json(self) -> str:
        '''
            Returns json string of the class
        '''
        return json.dumps(self.__dict__)


def get_api_token() -> str:
    '''
        Returns pikutis api token
    '''
    return settings.PIKUTIS_API_KEY

def call_pdf_api(file_string:str, api_params:Optional[PdfApiParams]) -> dict:
    '''
        Calls pikutis api and returns response
    '''
    api_url:str = "https://www.pikutis.lt/api/generate-pdf/"
    pdf_request:requests.Request = requests.Request('POST', url=api_url)
    pdf_request.headers = {"Token": get_api_token()}
    pdf_request.data = file_string.encode('utf-8')
    if api_params:
        pdf_request.params = api_params.to_json()

    pdf_response:requests.Response = requests.Session().send(pdf_request.prepare())
    return json.loads(pdf_response.content)

def get_pdf_file(pdf_response:dict) -> bytes:
    '''
        Returns pdf file bytes
    '''
    return base64.b64decode(pdf_response['file_bytes_base_64'])


from django.conf import settings

def fix_static_urls(html:str) -> str:
    """
    Fixes the static urls in the html file
    """
    return html.replace(settings.STATIC_URL, settings.SITE_URL + settings.STATIC_URL)

def fix_media_urls(html:str) -> str:
    """
    Fixes the media urls in the html file
    """
    return html.replace(settings.MEDIA_URL, settings.SITE_URL + settings.MEDIA_URL)